"""Wrappers for the dependency configuration files."""

import os
import logging

import yorm

from . import common
from . import git
from . import shell
from .exceptions import InvalidConfig, InvalidRepository, UncommittedChanges


log = logging.getLogger(__name__)


@yorm.attr(repo=yorm.types.String)
@yorm.attr(dir=yorm.types.String)
@yorm.attr(rev=yorm.types.String)
@yorm.attr(link=yorm.types.String)
class Source(yorm.types.AttributeDictionary):
    """A dictionary of `git` and `ln` arguments."""

    DIRTY = '<dirty>'
    UNKNOWN = '<unknown>'

    def __init__(self, repo, name, rev='master', link=None):
        super().__init__()
        self.repo = repo
        self.dir = name
        self.rev = rev
        self.link = link
        if not self.repo:
            raise InvalidConfig("'repo' missing on {}".format(repr(self)))
        if not self.dir:
            raise InvalidConfig("'dir' missing on {}".format(repr(self)))

    def __repr__(self):
        return "<source {}>".format(self)

    def __str__(self):
        fmt = "'{r}' @ '{v}' in '{d}'"
        if self.link:
            fmt += " <- '{s}'"
        return fmt.format(r=self.repo, v=self.rev, d=self.dir, s=self.link)

    def __eq__(self, other):
        return self.dir == other.dir

    def __ne__(self, other):
        return self.dir != other.dir

    def __lt__(self, other):
        return self.dir < other.dir

    def update_files(self, force=False, fetch=False, clean=True):
        """Ensure the source matches the specified revision."""
        log.info("Updating source files...")

        # Enter the working tree
        if not os.path.exists(self.dir):
            log.debug("Creating a new repository...")
            git.clone(self.repo, self.dir)
        shell.cd(self.dir)

        # Check for uncommitted changes
        if not force:
            log.debug("Confirming there are no uncommitted changes...")
            if git.changes(include_untracked=clean):
                common.show()
                msg = "Uncommitted changes: {}".format(os.getcwd())
                raise UncommittedChanges(msg)

        # Fetch the desired revision
        if fetch or self.rev not in (git.get_branch(),
                                     git.get_hash(),
                                     git.get_tag()):
            git.fetch(self.repo, self.rev)

        # Update the working tree to the desired revision
        git.update(self.rev, fetch=fetch, clean=clean)

    def create_link(self, root, force=False):
        """Create a link from the target name to the current directory."""
        if self.link:
            log.info("Creating a symbolic link...")
            target = os.path.join(root, self.link)
            source = os.path.relpath(os.getcwd(), os.path.dirname(target))
            if os.path.islink(target):
                os.remove(target)
            elif os.path.exists(target):
                if force:
                    shell.rm(target)
                else:
                    common.show()
                    msg = "Preexisting link location: {}".format(target)
                    raise UncommittedChanges(msg)
            shell.ln(source, target)

    def identify(self, allow_dirty=True, allow_missing=True):
        """Get the path and current repository URL and hash."""
        if os.path.isdir(self.dir):

            shell.cd(self.dir)

            path = os.getcwd()
            url = git.get_url()
            if git.changes(display_status=not allow_dirty, _show=True):
                revision = self.DIRTY
                if not allow_dirty:
                    common.show()
                    msg = "Uncommitted changes: {}".format(os.getcwd())
                    raise UncommittedChanges(msg)
            else:
                revision = git.get_hash(_show=True)
            common.show(revision, log=False)

            return path, url, revision

        elif allow_missing:

            return os.getcwd(), '<missing>', self.UNKNOWN

        else:

            path = os.path.join(os.getcwd(), self.dir)
            msg = "Not a valid repository: {}".format(path)
            raise InvalidRepository(msg)

    def lock(self):
        """Return a locked version of the current source."""
        _, _, revision = self.identify(allow_missing=False)
        source = self.__class__(self.repo, self.dir, revision, self.link)
        return source
